@@ -44,4 +44,9 @@ class MemoryStorageBase(ABC):
     @abstractmethod
     async def clear(self, user_id: str) -> None:
         """Clear all memories for a user."""
+        pass
+
+    @abstractmethod
+    async def delete(self,memory_ids: List[str]):
+        """Delete memories by their IDs."""
         pass
\ No newline at end of file
