@@ -4,7 +4,7 @@ from typing import Dict, List, Optional, Tuple, Union
 
 # Local imports
 from .base_messages import MessageStorageBase
-from ...schemas import Message
+from ...schemas import Message,MessageType
 
 
 class MessageStorageLocalConfig:
@@ -201,7 +201,7 @@ class MessageStorageLocal(MessageStorageBase):
 
     def _is_tool_message(self, message: Message) -> bool:
         """Check if a message is a tool-related message."""
-        return bool(getattr(message, 'tool_call', None))
+        return message.type in {MessageType.FUNCTION_CALL, MessageType.FUNCTION_CALL_OUTPUT}
 
 
 
